import re
import pandas
from itertools import groupby
from collections import namedtuple, defaultdict

def tree(): return defaultdict(tree)

class BasicBlock(object):
    def __init__(self, label, llvm_name, lineno, endlineno=-1, codepoints=None):
        self.__dict__.update(locals())
        self.codepoints = []

    def addcp(self, codepoint):
        self.codepoints.append(codepoint)

    def end(self, lineno):
        self.endlineno = lineno

    # Need to come up with an algo...
    def contains(self, codepoint):
        pass

    def __repr__(self):
        s = 'BasicBlock(label="%s", llvm_name="%s", lineno=%d, endlineno=%d, codepoints='%(self.label, self.llvm_name, self.lineno, self.endlineno)
        s += self.codepoints.__repr__()
        s += ')'
        return s

# .file   1 "/data/src/parsec-2.1-patch/pkgs/apps/blackscholes/obj/amd64-linux.ptprofile-active/blackscholes.m4.cpp"
re_source = re.compile(r'\.file\s+(?P<fileno>\d+)\s+"(?P<filepath>[^"]+)"')
# for re.search (research! ;))

# .loc    1 302 0
re_cp     = re.compile(r'\.loc\s+(?P<fileno>\d+)\s+(?P<row>\d+)\s+(?P<col>\d+)')
# for re.search

codepoint = namedtuple('codepoint', ['fileno', 'line', 'col'])

def makecp(match):
    params = tuple(int(x) for x in match.groups())
    return codepoint(*params)

# # BB#1:                                 # %_Z9bs_threadPv:1
# .LBB0_17:                               # %_Z9bs_threadPv:15
re_bb     = re.compile(r'^(?P<label>[^:]+):\s+# %(?P<llvm_name>.+)$')
# for re.match (we use ^ and $)

class GrowingList(list):
    def __setitem__(self, i, v):
        if i >= len(self):
            self.extend([None] * (i + 1 - len(self)))
        list.__setitem__(self, i, v)

def make_llvm_bb_dict(bbs):
    return dict((bb.llvm_name, bb) for bb in bbs)

#TODO: hdf5! (or don't bother)
asmfile = namedtuple('asmfile', [ 'files', 'bbs', 'llvm_bb_dict' ])
def make_asmfile(files, bbs):
    llvm_bb_dict = make_llvm_bb_dict(bbs)
    return asmfile(files, bbs, llvm_bb_dict)

def parse(f):
    """Returns (paths, bbs), both arrays. files is a list of paths given
    by the debug information in the asm file f, and bbs is a list of
    BasicBlocks in order."""
    paths = GrowingList()
    bbs = []
    for lineno, line in enumerate(f):
        # This re_x.search(y) is more efficient--avoids a call to compile which
        # does nothing but typecheck.
        m = re_source.search(line)
        if m:
            file_no  = int(m.group('fileno'))
            filepath = m.group('filepath')
            if filepath is None:
                continue
            paths[file_no] = filepath
            continue

        m = re_cp.search(line)
        if m and len(bbs) > 0:
            bbs[-1].addcp(makecp(m))
            continue

        m = re_bb.match(line)
        if m:
            if len(bbs) > 0:
                bbs[-1].end(lineno - 1)

            label = m.group('label')
            llvm_name = m.group('llvm_name')
            bbs.append(BasicBlock(label, llvm_name, lineno))
            continue

        if line.find('.Ltext_end') > 0:
            # If we encounter this label, there is no more code to see.
            bbs[-1].end(lineno - 1)
            break

    return make_asmfile(paths, bbs)

def codepoint_cmp(c1, c2):
    if c1.fileno != c2.fileno:
        raise Exception("codepoints must be from the same file!")
    dl = c1.line - c2.line
    if dl == 0:
        return c1.col - c2.col
    else:
        return dl

# Uses pandas!
def codepoints_dataframe(asmfile):
    rows = []
    for bb in asmfile.bbs:
        for cp in bb.codepoints:
            f = asmfile.files[cp.fileno]
            row = { 'file': f, 'line': cp.line, 'col': cp.col, 'bb': bb.llvm_name }
            rows.append(row)

    df = pandas.DataFrame(rows)
    return df

if __name__ == '__main__':
    import argparse
    from pprint import pprint
    parser = argparse.ArgumentParser(description='Test the asm parser.')
    parser.add_argument('src', help='the asm file')
    args = parser.parse_args()

    with open(args.src) as f:
        parsed_asm = parse(f)
        pprint(parsed_asm)

