import argparse
import pandas
import re
import os
import numpy as np
from glob import glob

import zickle
from profile import load_profile
from asmparse import parse

import pdb

parser = argparse.ArgumentParser(description='Compile experiment output on the filesystem to a database.')
parser.add_argument('inpath', help='path to the experiment output (input to this program)')
parser.add_argument('outpath', help='path to the collated output (output of this program; pickle)')
parser.add_argument('-v', action='store_true', help='verbose (debug)', default=False)

V = False

def strip_suffix(outfile):
    outfile = os.path.basename(outfile)
    num = outfile[:outfile.find('.')]
    return int(num)

def jaccard(a, b):
    return len(a & b) / float(len(a | b))

# real    0m0.482s
re_time_line = re.compile(r'\w+\s+(?P<min>\d+)m(?P<sec>[\d:]+)s')
def parse_time_line(line):
    m = re_time_line.search(line)
    if m is None: return np.nan
    return 60 * float(m.group('min')) + float(m.group('sec'))

def parse_time(path_to_log, ntrials):
    times = np.zeros((ntrials, 3))
    ct = -1
    with open(path_to_log) as f:
        for line in f:
            if line.find('real') > 0:
                ct += 1
                times[ct][0] = parse_time_line(line)
            elif line.find('user') > 0:
                times[ct][1] = parse_time_line(line)
            elif line.find('sys') > 0:
                times[ct][2] = parse_time_line(line)

    index = range(1, ntrials + 1) # the trials are 1-indexed
    return pandas.DataFrame(times, index=index, columns=('real', 'user', 'sys'))

def load_app(path, theta_stupid=0.8):
    name = os.path.basename(path)
    # Collate the datasets.
    prefix = '%s/'%path
    profile_paths = glob(prefix + '*.out')
#    pdb.set_trace()
    profiles_dict = dict((strip_suffix(p), load_profile(p)) for p in profile_paths)

    if len(profiles_dict) == 0:
        print '[%s] Error: no profiles.'%name
        return None
#    pdb.set_trace()

    if V: print "Looking in " + path

#    pdb.set_trace()
    profiles_panel = pandas.Panel.from_dict(profiles_dict)
#    pdb.set_trace()

    # Load all of the .asm files and find the right one--the one that
    # matches the one the datasets.

    # Parse timing information (parsec only)
    times = np.empty
    logpath = prefix + 'LOG'
    if os.path.exists(logpath):
        times = parse_time(logpath, profiles_panel.shape[0])

    # The index of each profile is our set of LLVM basic blocks. The only
    # problem is that the function name is prepended here.
    stripped_names = ( s[s.find(':')+1:] for s in profiles_panel.major_axis )
    llvm_bb_names = set(stripped_names)
    asm_paths = glob(prefix + '*.s')
    if len(asm_paths) > 0:
        def asmfile_key(asm_file):
            names = set(asm_file.llvm_bb_dict.iterkeys())
            return jaccard(llvm_bb_names, names)

    #    pdb.set_trace()
        def open_asm(p):
            with open(p) as f:
                return parse(f)
        asm_file = max(( open_asm(p) for p in asm_paths ), key=asmfile_key)
        # Sanity/Debug Check
    #    pdb.set_trace()
        print '[%s] asm file has jaccard %g'%(name, asmfile_key(asm_file))
    else:
        asm_file = None

    return { 'profiles': profiles_panel, 'times': times, 'asm_file': asm_file }

if __name__ == '__main__':
    args = parser.parse_args()
    V = args.v

    def kv(app):
        parsed = load_app(os.path.join(args.inpath, app))
        if parsed is None:
            return None
        return (app, parsed)

    out_dict = dict(filter(None, map(kv, os.listdir(args.inpath))))
    zickle.dump(out_dict, args.outpath)

