#!/usr/bin/env python
# -*- coding: utf-8 -*-

# stolen from http://www.kenjisato.jp/blog/2011/03/28/scatterplot-matrix-with-matplotlib/
# adapted for DataFrame
import matplotlib.pyplot as plt

def pairs(data, labels=None, fontsize=12, nbins=10):
    """Makes a scatterplot matrix:
    Inputs:
      data - a DataFrame object, or an ndarray
      labels - if data is an ndarray, column labels.

    Output:
      fig - matplotlib.figure.Figure Object
    """

    N = len(data)
    if labels is None and hasattr(data, 'index'):
        labels = data.index

    fig = plt.figure()

    for i in range(N):
        for j in range(N):
            ax = fig.add_subplot(N,N,i*N+j+1)

            if j == 0: ax.set_ylabel(labels[i],size=fontsize)
            if i == 0: ax.set_title(labels[j],size=fontsize)
            if i == j:
                ax.hist(data[i], nbins)
            else:
                ax.scatter(data[j], data[i])

    return fig

# Example
if __name__ == "__main__":
    import numpy as np
    import numpy.random as npr

    X = npr.randn(100)
    Y = 1.2 * X + npr.normal(0.0, 0.1, 100)
    Z = - Y ** 2 + X + 0.05 * npr.random(100)
    W = X + Y - Z + npr.normal(0.0, 2.0, 100)

    data = [X, Y, Z, W]
    data_name = ['Data X', 'Data Y', 'Data Z', 'Data W']

    fig = scatterplot(data, data_name)

    fig.savefig('scatterplot.png', dpi=120)
    plt.show()
