import numpy as np
import pandas
import struct
import sys
import gzip

def load_profile(path, verbose=False, strip=True):
    """strip will remove trailing all-zero columns (threads never got that high)"""
    if path.endswith(".gz"):
        pf = gzip.open(path, 'r');
    else:
        pf = open(path, 'r');

    nbbs, nthreads, profile_type = struct.unpack('III', pf.read(12))
    profile_n = nthreads * nbbs
    profile = (np.fromfile(pf, dtype='I', count=profile_n)
                 .reshape(nthreads, nbbs)
                 .transpose())
    bb_names = pf.read().split("\0")
    if len(bb_names) != nbbs:
        print >> sys.stderr, ("profile reported %d basic blocks but"
            "only %d names were read"%(nbbs, len(bb_names)))
        return None

    pf.close()

    if strip:
        _, c = profile.shape
        c -= 1
        while np.count_nonzero(profile[:,c]) == 0:
            c -= 1
        # Now column c is the last nonzero column. :(c+1) includes 0 to c
        # inclusive.
        profile = profile[:,:(c+1)]

    df = pandas.DataFrame(profile, index=bb_names)
    if verbose:
        print bb_names
        print df.head(2)

    return df
