// This microbenchmark attempts to distinguish buggy basic blocks from merely
// hot blocks.
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <pthread.h>

#define NITERS 1000

static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
// Invariant: protect + naked == 0
static int protected, naked;
static __thread int unrelated;
static int bugs, checks;

typedef void *start_routine_t(void *);

// This has hot blocks that are also racy, but correct.
void monitor() {
  while (1) {
    pthread_mutex_lock(&lock);
    checks++;
    if (protected + naked != 0) {
      bugs++;
    }
    pthread_mutex_unlock(&lock);
  }
}

void naked_to_protected() {
  int i;
  for (i = 0; i < NITERS; i++) {
    // This hot block is both racy and buggy.
    naked--;
    pthread_mutex_lock(&lock);
    protected++;
    pthread_mutex_unlock(&lock);
  }

  for (i = 0; i < NITERS; i++) {
    // This hot block is NOT racy or buggy.
    unrelated++;
  }
}

int main (int argc, char const* argv[]) {
  if (argc < 2) {
    fprintf(stderr, "Usage: %s NTHREADS\n", argv[0]);
    return 1;
  }

  int nthreads = atoi(argv[1]);
  if (nthreads <= 0) {
    fprintf(stderr, "Error: NTHREADS must be positive.\n");
    return 2;
  }

  int i;
  pthread_t monitor_thread;
  pthread_create(&monitor_thread, NULL, (start_routine_t*) monitor, NULL);

  pthread_t *threads = malloc(nthreads * sizeof(pthread_t));
  for (i = 0; i < nthreads; i++) {
    pthread_create(&threads[i], NULL, (start_routine_t*) naked_to_protected, NULL);
  }

  for (i = 0; i < nthreads; i++) {
    pthread_join(threads[i], NULL);
  }

  pthread_cancel(monitor_thread);
  printf("Bugs: %d/%d = %g\n", bugs, checks, bugs / (double) checks);

  return 0;
}

